/****************************************************************************
*  Program/file: AppDhcpd.c
*   
*  Copyright (C) by RTX A/S, Denmark.
*  These computer program listings and specifications, are the property of 
*  RTX A/S, Denmark and shall not be reproduced or copied or used in 
*  whole or in part without written permission from RTX A/S, Denmark.
*
*  Programmer: LKA
*
*  MODULE:
*  CONTROLLING DOCUMENT:
*  SYSTEM DEPENDENCIES:
*   
*   
*  DESCRIPTION: Implementation of very simple DNS server to be used with SoftAp
*               mode. It return one ip (AP)
*   
****************************************************************************/

/****************************************************************************
*                                  PVCS info                                  
*****************************************************************************

$Author:   Jordi Casas  $
$Date:   28 Apr 2015 07:19:38  $
$Revision:   1.0  $
$Modtime:   28 Apr 2015 07:19:38  $
$Archive:   ./Projects/Amelie/COLApps/Apps/SmartCitizenRTX4100/rtx_AppDns.h  $

*/

/**
 @file rtx_AppDns.c
 @brief Very simple DNS Server
 @author Jordi Casas
 @date 4/2015
*/


/****************************************************************************
*                               Include files                                 
****************************************************************************/
//Type definitions
#include <Core/RtxCore.h> // Mandatory, Must be the first include

//Framework/Kernel
#include <ROS/RosCfg.h>

//Interfaces
#include <Api/Api.h>
#include <Cola/Cola.h>
#include <SwClock/SwClock.h>
#include <Protothreads/Protothreads.h>
#include <NetUtils/NetUtils.h>
#include <PtApps/AppSocket.h>
#include <PtApps/AppCommon.h>

//Configurations

//Private 
#include <RtxEai/RtxEai.h>
#include <../Projects/Amelie/COLApps/Apps/SmartCitizenRTX4100/rtx_AppDns.h>
#include <PtApps/AppWiFi.h>

/****************************************************************************
*                              Macro definitions                              
****************************************************************************/
#define DNS_SERVER_PORT 53
#define printf(...) RtxEaiPrintf(ColaIf->ColaTaskId,__VA_ARGS__)

/****************************************************************************
*                     Enumerations/Type definitions/Structs                   
****************************************************************************/

/****************************************************************************
*                            Global variables/const                           
****************************************************************************/

/****************************************************************************
*                            Local variables/const                            
****************************************************************************/

/****************************************************************************
*                          Local Function prototypes                          
****************************************************************************/

/**
 * @brief Helper function to reverse data order
 * @param data : pointer to data
 **/
static void reverse_rsuint32(rsuint32* data) {
 rschar tmp_c[4];
 rsuint32 tmp = 0;

 memcpy(&tmp_c, data, 4);

 rsuint8 i;
 for (i=0; i<4; i++) {
  tmp = tmp << 8;
  tmp += tmp_c[i];
 }
*data = tmp;
}

/****************************************************************************
*                                Implementation                               
****************************************************************************/
/**
 * @brief Protothread used to handle the data received.
 * @param Pt : pointer to protothread (for yield it)
 * @param Mail : pointer to mails
 **/
static char PtOnDNSreq(struct pt *Pt, const RosMailType* Mail)
//static PT_THREAD(PtOnDNSreq(struct pt *Pt, const RosMailType* Mail))
{
    static DnsType *pRx;
    static DnsType *pTx;
    AppSocketDataType *pInst = (AppSocketDataType*)PtInstDataPtr;
    static ApiSocketAddrType client_addr;
  
    PT_BEGIN(Pt);

        while (AppWifiIsSoftAp()){
            //wait request
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_RECEIVE_FROM_IND) 
                    && ((ApiSocketReceiveFromIndType*)Mail)->Handle == pInst->SocketHandle);

            pRx = (DnsType*)((ApiSocketReceiveFromIndType*)Mail)->BufferPtr;
            client_addr = ((ApiSocketReceiveFromIndType*)Mail)->Addr;
            
                
            //request a buffer for the response
            SendApiSocketGetTxBufferReq(COLA_TASK, pInst->SocketHandle, sizeof(DnsType));
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_GET_TX_BUFFER_CFM) && 
                    ((ApiSocketGetTxBufferCfmType*)Mail)->Handle == pInst->SocketHandle);
            pTx = (DnsType*)((ApiSocketGetTxBufferCfmType*)Mail)->BufferPtr;
    
    
            //generate response
            //headers
            pTx->id = pRx->id;
            pTx->qdcount = pRx->qdcount;
            memcpy(&pTx->flags, dns_h_flags, 2);
            memcpy(&pTx->ancount, dns_h_ancount, 2);
            memcpy(&pTx->nscount, dns_h_nscount, 2);
            memcpy(&pTx->arcount, dns_h_arcount, 2);
            rsuint8 sizeTxbuffer = 12; //size headers
            
            //bodysection 
            rsuint8 body_size = 0;
            
            //question (bodysection)
            body_size = strlen((const char*)pRx->bodysection); //length domain string
            body_size += (1 + 4);	//1bytes for null bit + 2bytes fix
            memcpy(pTx->bodysection, pRx->bodysection, body_size);

            //Answer (bodysection)
            memcpy(pTx->bodysection+body_size, dns_a_name, 2); body_size +=2;
            memcpy(pTx->bodysection+body_size, dns_a_type, 2); body_size +=2;
            memcpy(pTx->bodysection+body_size, dns_a_class, 2); body_size +=2;
            memcpy(pTx->bodysection+body_size, dns_a_ttl, 4); body_size +=4;
            memcpy(pTx->bodysection+body_size, dns_a_sizeaddr, 2); body_size +=2;
    
            rsuint32 local_ip = (rsuint32)AppWifiIpv4GetAddress();
            reverse_rsuint32(&local_ip);
            memcpy(pTx->bodysection+body_size, &local_ip, 4); body_size +=4;    
                        
            //for debug purpose...
            rsuint8 p = 0; //first byte = first '.'
            printf("DNS: ");
            rschar c;
            
            while (1) {
                p++;
                memcpy(&c, pRx->bodysection+p, 1);
            
                if (c > 47)             //if is char, print it
                    printf("%c", c);
                
                else if ( c == 0)       //null = end of name
                    break;
                
                else printf(".");       //else .
            }
            printf("\n\n");
            
            
            //Sending data... 
            SendApiSocketSendToReq(COLA_TASK, pInst->SocketHandle, (rsuint8*)pTx, 
                    sizeTxbuffer + body_size, 0, client_addr);
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM));
            
            //Free buffers
            SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, (rsuint8*)pRx);
        }
    PT_END(Pt);
}

/**
 * @brief Init function for DNS Server
 * @param PtList : pointer to Protothread control
 **/
PtEntryType* rtx_AppDnsServer(RsListEntryType *PtList)
{
    ApiSocketAddrType addr;

    addr.Domain = ASD_AF_INET;
    addr.Ip.V4.Addr = 0;
    addr.Port = DNS_SERVER_PORT;
    
    printf ("DNS: Started\n\n");
    
    return AppSocketStartUdpServer(PtList, addr, PtOnDNSreq);   
}
