/****************************************************************************
*  Program/file: rtx_AppWeb.c
*   
*  Copyright (C) by RTX A/S, Denmark.
*  These computer program listings and specifications, are the property of 
*  RTX A/S, Denmark and shall not be reproduced or copied or used in 
*  whole or in part without written permission from RTX A/S, Denmark.
*
*  Programmer: LKA
*
*  MODULE:
*  CONTROLLING DOCUMENT:
*  SYSTEM DEPENDENCIES:
*   
*   
*  DESCRIPTION: Implementation of simple web server to be used with SoftAp
*               mode.
*   
****************************************************************************/

/****************************************************************************
*                                  PVCS info                                  
*****************************************************************************

$Author:   Jordi Casas  $
$Date:   28 Apr 2015 07:19:38  $
$Revision:   1.0  $
$Modtime:   28 Apr 2015 07:19:38  $
$Archive:   ./Projects/Amelie/COLApps/Apps/SmartCitizenRTX4100/rtx_AppWeb.c  $

*/

/**
 @file rtx_AppWeb.c
 @brief Simple Web Server
 @author Jordi Casas
 @date 4/2015, 5/2015, 6/2015
*/

/****************************************************************************
*                               Include files                                 
****************************************************************************/
//Type definitions
#include <Core/RtxCore.h> // Mandatory, Must be the first include

//Framework/Kernel
#include <ROS/RosCfg.h>

//Interfaces
#include <Api/Api.h>
#include <Cola/Cola.h>
#include <SwClock/SwClock.h>
#include <Protothreads/Protothreads.h>
#include <NetUtils/NetUtils.h>
#include <PtApps/AppSocket.h>
#include <PtApps/AppCommon.h>

//Configurations

//Private 
#include <../Projects/Amelie/COLApps/Apps/SmartCitizenRTX4100/rtx_AppWeb.h>
#include <PtApps/AppWiFi.h>
#include <RtxEai/RtxEai.h>
#include <stdio.h>
#include <Drivers/DrvSpiSlave.h>
#include <em_gpio.h>
#include <Drivers/DrvIntTemp.h>
#include <string.h>

/****************************************************************************
*                              Macro definitions                              
****************************************************************************/
#define printf(...) RtxEaiPrintf(ColaIf->ColaTaskId,__VA_ARGS__)
#define printc(...) RtxEaiLogComment(ColaIf->ColaTaskId,__VA_ARGS__)
/****************************************************************************
*                     Enumerations/Type definitions/Structs                   
****************************************************************************/

/****************************************************************************
*                            Global variables/const                           
****************************************************************************/

//Timers
static const RosTimerConfigType PacketDelayTimer =
  ROSTIMER(COLA_TASK, APP_PACKET_DELAY_TIMEOUT,
  APP_PACKET_DELAY_TIMER);
/****************************************************************************
*                            Local variables/const                            
****************************************************************************/
static AppDataWebType AppData;

/****************************************************************************
*                          Local Function prototypes                          
****************************************************************************/

/****************************************************************************
*                                Implementation                               
****************************************************************************/
/**
 * @brief Set ptr_cmd to ptr of cmd (if exist)
 **/
void find_var_cmd() {
    rsuint8 c=1;
    
    while (AppData.ptr_cmd != NULL){
            AppData.ptr_cmd = (rsuint8*)strstr((rschar*)AppData.ptr_cmd, web_optioncmd);
        
            if (AppData.ptr_cmd == NULL) break;
        
            if ((*(AppData.ptr_cmd-1) == 0x00) || (*(AppData.ptr_cmd-1) == '&') || (c==1) ) {
                //if exist, find end
                rsuint8 *ptr_test = (rsuint8*)strchr((rschar*)AppData.ptr_cmd,'&');
                if (ptr_test != NULL) memset (ptr_test, 0x00, 1);
                break;
            }
            else {
                AppData.ptr_cmd++;
                c ++;
            }
        }
}

/**
 * @brief Get method & url from petition
 * @param ptr_data: Pointer to data (web request)
 **/
void parser_webdata(rsuint8* ptr_data) {
    AppData.ptr_url=NULL;AppData.ptr_cmd=NULL;
    
    AppData.ptr_url = (rsuint8*)strchr((rschar*)ptr_data, ' ')+1;     //url
    memset (AppData.ptr_url-1, 0x00, 1);                              //end for method
    
    AppData.ptr_cmd = (rsuint8*)(strchr((rschar*)AppData.ptr_url, ' '));
    memset (AppData.ptr_cmd, 0x00, 1);                                //end for url
    
    //save method
    if (memcmp(ptr_data, "GET", strlen((rschar*)ptr_data)) == 0) AppData.method = GET;
    else if (memcmp(ptr_data, "POST", strlen((rschar*)ptr_data)) == 0) AppData.method = POST;
    else AppData.method = UNK;
    
    
    if (AppData.method == GET) {
        //exist any var?
        AppData.ptr_cmd = (rsuint8*)(strchr((rschar*)AppData.ptr_url, '?'));
        if (AppData.ptr_cmd != NULL) {
            memset(AppData.ptr_cmd, 0x00, 1);
            AppData.ptr_cmd += 1;
            find_var_cmd();
        }  
    }
}

/**
 * @brief Protothread used to download file from arduino and send it to client
 * @param Pt : pointer to protothread (for yield it)
 * @param Mail : pointer to mails
 * @param pInst: Pointer to instance (socket connection)
 * @param url: Pointer to file
 **/
static char Pt_SPI_getfile(struct pt *Pt, const RosMailType *Mail, AppSocketDataType *pInst, rsuint8 *file) {
//static PT_THREAD(Pt_SPI_getfile(struct pt *Pt, const RosMailType *Mail, AppSocketDataType *pInst, rsuint8 *file)) {
    static struct pt childPt;
    static rsuint32 file_size;
    static rsuint16 buffer_spi;
    static rsuint32 rest_file;
    
    PT_BEGIN(Pt); 
        printf("HTTPD.SPI.File: %s\n", file);
        //Prepare data (api + path)
        file_size = 0; buffer_spi = 0;
        memset(AppData.TX_Ptr, 0xA1, 1);
        memset(AppData.TX_Ptr+1, strlen((rschar*)file), 1);
        memcpy(AppData.TX_Ptr+2, file, strlen((rschar*)file));
     
        PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveInit(&childPt, Mail, SPI_baudrate));
        DrvSpiSlaveRxFlush();

        DrvSpiSlaveTxStart(AppData.TX_Ptr, strlen((rschar*)file)+2);

        //shutdown led (interrupt -> arduino read spi)
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 1); // enviamos ruta del archivo

        //timeout (spi response)
        RosTimerStart(APP_PACKET_DELAY_TIMER, (3 * RS_T1SEC), &PacketDelayTimer);

        PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_TX_DONE) || IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
        if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
            PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template, "500 Internal Server Error", "ERROR", "SPI TIMEOUT");
            SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
            printf("HTTPD.SPI.File: TIMEOUT\n");
            PT_EXIT(Pt);
        }
        
        DrvSpiSlaveRxFlush();
        
        //poweron led (data sended)
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0); //path sended
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_DATA));

        //1 byte = qty bytes for store file length (1-5)
        rsuint8 num_c, long_file[5];
        DrvSpiSlaveRx(&num_c,1);
        
        if (num_c == 0) { //file not exist -> send 404 and close
            //close SPI
            PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
            
            //prepare response
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template, "404 Not Found", "File not exist or SD not initializated", web_error404);
            
            //send it
            SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            printf("HTTPD.SPI: File not found (404 sended)\n\n");
            PT_EXIT(Pt);
        }
        
        DrvSpiSlaveRx(&long_file[0],num_c);
        
        //Convert array of bytes in long number
        for ( ; num_c > 0; num_c --) {
            file_size <<= 8;
            file_size += long_file[num_c-1];
        }
       
        
        /*************************
        **Negotiate buffer (SPI)**
        *************************/
        
        //Negotiate buffersize [2bytes]
        buffer_spi = RSMIN(DrvSpiSlaveGetBufferSize(), maxBufferTX);
        
        //bytes to receive
        for (num_c = 0; num_c < 2; num_c ++) {
            memset(AppData.TX_Ptr+num_c, buffer_spi % 256, 1);
            buffer_spi >>= 8;
        }
        
        //send our spi buffer
        DrvSpiSlaveTxStart(AppData.TX_Ptr, 2);
        
        //shutdown led (interrupt -> arduino read spi)
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 1);    // enviamos nuestro buffer
       
        //wait until receive 2 interrupts (2 bytes)
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_DATA));
        DrvSpiSlaveRxFlush();
        
        //poweron led (data sended)
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);    //buffer enviado

        
        //wait arduino send buffer (2bytes)
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_DATA));
        DrvSpiSlaveRx(&long_file[0], 2);
        
        //bytes to receive
        buffer_spi = 0;
        for (num_c = 2; num_c > 0; num_c --) {
            buffer_spi <<= 8;
            buffer_spi += long_file[num_c-1];
        }        
        
        /***************
        **Request file**
        ***************/
        
        //send simple header to client (web_msg_template_file)
        AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_file, (int)file_size);
        //send header
        SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);            
        
        //count number chunks
        num_c = 1; 
        while (file_size > buffer_spi) {
            file_size -= buffer_spi;
            num_c ++;
        }
        rest_file = file_size;
        file_size = num_c;
//printf("HTTPD.SPI.File: chunks: %i buffer: %i length_last_file: %i \n", file_size, buffer_spi, rest_file);
        
        //loop chunks file (LAST CHUNK EXCLUDED)
        AppData.TX_Length = buffer_spi;
        DrvSpiSlaveSetTrigger(AppData.TX_Length);
        for (; file_size > 1; file_size -- ) {
            //Timer
            RosTimerStart(APP_PACKET_DELAY_TIMER, (3 * RS_T1SEC), &PacketDelayTimer);
            
            
            DrvSpiSlaveRxFlush();
            GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 1); //poweroff
            
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_TRIGGER) || IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
            if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
                PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
                RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
                GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
                printf("HTTPD.SPI.File.Chunk: TIMEOUT\n");
                PT_EXIT(Pt);
            }
            RosTimerStop(APP_PACKET_DELAY_TIMER);
            
            DrvSpiSlaveRx(AppData.TX_Ptr,AppData.TX_Length);
            //send chunk
            SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);            
            
            GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0); //poweron
        }
        
        //receive last chunk
        //Timer
        RosTimerStart(APP_PACKET_DELAY_TIMER, (3 * RS_T1SEC), &PacketDelayTimer);
        
        DrvSpiSlaveRxFlush();
        DrvSpiSlaveSetTrigger(rest_file);
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 1); //poweroff
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_TRIGGER) || IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
        if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
            PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
            printf("HTTPD.SPI.File.Last: TIMEOUT\n");
            PT_EXIT(Pt);
        }
        RosTimerStop(APP_PACKET_DELAY_TIMER);
        
        AppData.TX_Length = rest_file;
        DrvSpiSlaveRx(AppData.TX_Ptr, AppData.TX_Length);
             
        //send last chunk
        SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0); //poweron
        
        //close SPI
        PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
        
        RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);        
        printf("HTTPD.SPI: File Sended\n\n");
    PT_END(Pt);
}

/**
 * @brief Protothread used to send command to arduino, get result and send it to client
 * @param Pt : pointer to protothread (for yield it)
 * @param Mail : pointer to mails
 * @param pInst: Pointer to instance (socket connection)
 * @param cmd: Pointer to command
 **/
static char Pt_SPI_sendcmd(struct pt *Pt, const RosMailType *Mail, AppSocketDataType *pInst, rsuint8 *cmd) {
//static PT_THREAD(Pt_SPI_sendcmd(struct pt *Pt, const RosMailType *Mail, AppSocketDataType *pInst, rsuint8 *cmd)) {
    static struct pt childPt;
    static rsuint8 num_c;
    
    PT_BEGIN(Pt);  
    printf("HTTPD.SPI.CMD: %s\n", cmd);
    
    //Prepare data
    memset(AppData.TX_Ptr, 0xA0, 1);
    memset(AppData.TX_Ptr+1, strlen((rschar*)cmd), 1);
    memcpy(AppData.TX_Ptr+2, cmd, strlen((rschar*)cmd));
    
    PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveInit(&childPt, Mail, SPI_baudrate));
    DrvSpiSlaveRxFlush();
    
    DrvSpiSlaveTxStart(AppData.TX_Ptr, strlen((rschar*)cmd)+2);
    
    GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 1); //poweroff
    
    //Timer    
    RosTimerStart(APP_PACKET_DELAY_TIMER, (1 * RS_T1SEC), &PacketDelayTimer);
    
    PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_TX_DONE) || IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
    if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
        PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
        AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template, "500 Internal Server Error", "ERROR", "SPI TIMEOUT");
        SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
        RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
        printf("HTTPD.SPI.CMD: TIMEOUT\n");
        PT_EXIT(Pt);
    }
    
    DrvSpiSlaveRxFlush();
    
    //poweron led (data sended)
    GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
    
    PT_YIELD_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_DATA));
    
    //Wait 100ms (arduino sending data...)
    RosTimerStart(APP_PACKET_DELAY_TIMER, (100 * RS_T1MS), &PacketDelayTimer);
    PT_YIELD_UNTIL(Pt, IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT));
    
    //1 byte = qty bytes for chars
    num_c=0;
    DrvSpiSlaveRx(&num_c,1);
    
    //get response of cmd
    DrvSpiSlaveRx(&AppData.data[0], num_c);
    AppData.data[num_c] = 0x00;
    
    //close SPI
    PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveClose(&childPt, Mail));
    
    printf("HTTPD.SPI.CMD.R:%s\n",AppData.data);
    
    //prepare response
    AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_cmd, "200 OK", "CMD RESPONSE", &AppData.data[0]);
                    
    //send it
    SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
    PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
    RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
    printf("HTTPD.SPI.CMD Exit response(cmd)\n\n");
    PT_END(Pt);
}

/**
 * @brief Protothread used to execute the command and send result to client
 * @param Pt : pointer to protothread (for yield it)
 * @param Mail : pointer to mails
 * @param pInst: Pointer to instance (socket connection)
 * @param command: Pointer to command
 **/
static char PT_process_rtx_command(struct pt *Pt, const RosMailType* Mail, AppSocketDataType *pInst, rschar *command) {
//static PT_THREAD(PT_process_rtx_command(struct pt *Pt, const RosMailType* Mail, AppSocketDataType *pInst, rschar *command)) {
    static struct pt childPt;
    
    PT_BEGIN(Pt);
    printf("RTX.CMD: %s\n", AppData.ptr_cmd);
    
        if (strcmp(command, "getTemperature") == 0) {
            rsint16 Temperature;
            DrvIntTemp_Init();
            PT_SPAWN(Pt, &childPt, PtDrvIntTempMeasure(&childPt, Mail, &Temperature));
                
            rschar tmpchar[3];
            sprintf(&tmpchar[0],"%i",Temperature/10);
                
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_cmd, "200 OK", "Command for RTX(getTemperature)", &tmpchar[0]);
        } 
        else if (strcmp(command, "getAPconf") == 0) {
            typedef struct {
                ApInfoType ap_info;
                rsuint8 use_dhcp;
                ApiSocketAddrType static_address, static_subnet, static_gateway;
            } AppDataType;    
            AppDataType app_data;

            NvsRead(NVS_OFFSET(Free), sizeof(AppDataType), (rsuint8 *)&app_data);
            
            if (!app_data.use_dhcp) {
                //ip
                if (app_data.static_address.Ip.V4.Addr == -1) {
                    sprintf((rschar*)AppData.RX_Ptr, "<br>IP: unconfigured<br>");
                } else {
                    char ip[15];
                    inet_ntoa(app_data.static_address.Ip.V4.Addr, ip);
                    sprintf((rschar*)AppData.RX_Ptr, "<br>IP: %s<br>", ip);
                }
            
                //mask
                if (app_data.static_subnet.Ip.V4.Addr == -1) {
                    sprintf((rschar*)&AppData.data[0], "mask: unconfigured<br>");
                } else {
                    char ip[15];
                    inet_ntoa(app_data.static_subnet.Ip.V4.Addr, ip);
                    sprintf((rschar*)&AppData.data[0], "mask: %s<br>", ip);
                }
                strcat ((rschar*)AppData.RX_Ptr, (rschar*)&AppData.data[0]);
            
                //gw
                if (app_data.static_gateway.Ip.V4.Addr == -1) {
                    sprintf((rschar*)&AppData.data[0], "gateway: unconfigured<br>");
                } else {
                    char ip[15];
                    inet_ntoa(app_data.static_gateway.Ip.V4.Addr, ip);
                    sprintf((rschar*)&AppData.data[0], "gateway: %s<br>", ip);
                }
                strcat ((rschar*)AppData.RX_Ptr, (rschar*)&AppData.data[0]);
            
            } else sprintf((rschar*)AppData.RX_Ptr, "<br>DHCP client enabled<br>");
                        
            //essid
            if (app_data.ap_info.Ssid[0] == 0xFF) {
                sprintf((rschar*)&AppData.data[0], "ssid: unconfigured<br>");
            } else {
                sprintf((rschar*)&AppData.data[0], "ssid: %s<br>", (rschar*)app_data.ap_info.Ssid);
            }
            strcat ((rschar*)AppData.RX_Ptr, (rschar*)&AppData.data[0]);
            
            //security
            ApiWifiSecTypeType sec = app_data.ap_info.SecurityType;
            switch (sec) {
                case AWST_NONE:
                    sprintf((rschar*)&AppData.data[0], "security: none<br>");
                    break;
                case AWST_WEP:
                    sprintf((rschar*)&AppData.data[0], "security: wep<br>");
                    break;
                case AWST_WPA:
                    sprintf((rschar*)&AppData.data[0], "security: wpa<br>");
                    break;
                case AWST_WPA2:
                    sprintf((rschar*)&AppData.data[0], "security: wpa2<br>");
                    break;
            }
            strcat ((rschar*)AppData.RX_Ptr, (rschar*)&AppData.data[0]);
            
            //key
            if (sec != AWST_NONE) {
                sprintf((rschar*)&AppData.data[0], "key: %s<br>", (rschar*)app_data.ap_info.Key );
                strcat ((rschar*)AppData.RX_Ptr, (rschar*)&AppData.data[0]);
            }
            
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_cmd, "200 OK", "CMD RESPONSE", AppData.RX_Ptr);
        }
        else if (strncmp(command, "setAPconf", 9) == 0) {
            //Get parameters
                /*1:IP
                 * 2:MASK
                 * 3:GW
                 * 4:ESSID
                 * 5:SECURITY
                 * 6:KEY
                 */
            
            rsuint8 *url_ip, *url_mask, *url_gw, *url_essid, *url_sec, *url_key;
            
            //Prepare parameters
            url_ip = (rsuint8*)strstr((rschar*)command, &web_cmd_separator[0]) + sizeof(web_cmd_separator)-1;
            
            url_mask = (rsuint8*)strstr((rschar*)url_ip, &web_cmd_separator[0]); 
                memset(url_mask,0x00,1); url_mask += sizeof(web_cmd_separator)-1;
            
            url_gw = (rsuint8*)strstr((rschar*)url_mask, &web_cmd_separator[0]); 
                memset(url_gw,0x00,1); url_gw += sizeof(web_cmd_separator)-1;
                
            url_essid = (rsuint8*)strstr((rschar*)url_gw, &web_cmd_separator[0]); 
                memset(url_essid,0x00,1); url_essid += sizeof(web_cmd_separator)-1;
                
            url_sec = (rsuint8*)strstr((rschar*)url_essid, &web_cmd_separator[0]); 
                memset(url_sec,0x00,1); url_sec += sizeof(web_cmd_separator)-1;
                
            url_key = (rsuint8*)strstr((rschar*)url_sec, &web_cmd_separator[0]); 
                memset(url_key,0x00,1); url_key += sizeof(web_cmd_separator)-1;
            
            //Prepare data
            typedef struct {
                ApInfoType ap_info;
                rsuint8 use_dhcp;
                ApiSocketAddrType static_address, static_subnet, static_gateway;
            } AppDataType;    
            
            //Get data 
            AppDataType app_data;
            NvsRead(NVS_OFFSET(Free), sizeof(AppDataType), (rsuint8 *)&app_data);
            
            //If exist IP, save it
            if (strlen((rschar*)url_ip) > 0) {
                inet_aton((rschar*)url_ip, &app_data.static_address.Ip.V4.Addr);
            }
            //If exist MASK, save it
            if (strlen((rschar*)url_mask) > 0) {
                inet_aton((rschar*)url_mask, &app_data.static_subnet.Ip.V4.Addr);
            }
            //If exist GW, save it
            if (strlen((rschar*)url_gw) > 0) {
                inet_aton((rschar*)url_gw, &app_data.static_gateway.Ip.V4.Addr);
            }
            
            //If exist essid, save it
            if (strlen((rschar*)url_essid) > 0) {
                strcpy((rschar*)&app_data.ap_info.Ssid[0], (rschar*)url_essid);
                app_data.ap_info.SsidLength = strlen((rschar*)url_essid);
            }
            
            /*Check key:
            possible values are:
             * 0:none
             * 1:wep
             * 2:wpa (tkip)
             * 3:wpa (aes)~ccmp
             * 4:wpa2 (tkip)
             * 5:wpa2 (aes)~ccmp
             */;
            app_data.ap_info.KeyIndex = 0;
            app_data.ap_info.KeyLength = 0;
            app_data.ap_info.Mcipher = 0;
            app_data.ap_info.Ucipher = 0;
            app_data.ap_info.SecurityType = AWST_NONE;
            
            if (strlen((rschar*)url_key) > 0) {
                app_data.ap_info.KeyLength = strlen((rschar*)url_key);
                strcpy((rschar*)&app_data.ap_info.Key[0], (rschar*)url_key);
            }
            
            switch (*url_sec) {
                case '1':
                    app_data.ap_info.SecurityType = AWST_WEP;
                    break;
                    
                case '2':
                    app_data.ap_info.SecurityType = AWST_WPA;
                    app_data.ap_info.Mcipher = AWCT_TKIP;
                    app_data.ap_info.Ucipher = AWCT_TKIP;
                    break;
                    
                case '3':
                    app_data.ap_info.SecurityType = AWST_WPA;
                    app_data.ap_info.Mcipher = AWCT_CCMP;
                    app_data.ap_info.Ucipher = AWCT_CCMP;
                    break;
                    
                case '4':
                    app_data.ap_info.SecurityType = AWST_WPA2;
                    app_data.ap_info.Mcipher = AWCT_TKIP;
                    app_data.ap_info.Ucipher = AWCT_TKIP;
                    break;
                    
                case '5':
                    app_data.ap_info.SecurityType = AWST_WPA2;
                    app_data.ap_info.Mcipher = AWCT_CCMP;
                    app_data.ap_info.Ucipher = AWCT_CCMP;
                    break;
            }
            
            //Save it
            NvsWrite(NVS_OFFSET(Free), sizeof(AppDataType), (rsuint8*)&app_data); 
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_cmd, "200 OK", "Command for RTX", "Config saved");
            
        }
        else {
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template_cmd, "200 OK", "Command for RTX", "Command unknown");
        }
        
        //Send it
        SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
        RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
        printf("HTTPD:Exit\n\n");
                
    PT_END(Pt);
}

/**
 * @brief Protothread used to handle the data received.
 * @param Pt : pointer to protothread (for yield it)
 * @param Mail : pointer to mails
 **/
static char PtOnHTTPreq(struct pt *Pt, const RosMailType* Mail)
//static PT_THREAD(PtOnHTTPreq(struct pt *Pt, const RosMailType* Mail))
{
    AppSocketDataType *pInst = (AppSocketDataType*)PtInstDataPtr;    
    static struct pt childPt;
    
    PT_BEGIN(Pt);
        GPIO_PinModeSet(gpioPortF, 2, gpioModePushPull, 0);
        RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_BUSY);
        RosTimerStart(APP_PACKET_DELAY_TIMER, (10 * RS_T1SEC), &PacketDelayTimer);

        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_RECEIVE_IND) || IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
        
        //if an unexpected packet is received kill it
        if (IS_RECEIVED(API_SOCKET_RECEIVE_IND) && ((ApiSocketReceiveIndType*)Mail)->Handle != pInst->SocketHandle) {
            printf("HTTPD: UNKNOWN CONNECTION. KILLING...\n\n");
            SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, ((ApiSocketReceiveIndType*)Mail)->BufferPtr);
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            PT_EXIT(Pt);
        }
        
        //if client doesnt send nothing (timeout) close connection
        if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
            printf("HTTPD: TIMEOUT\n");
            SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, AppData.RX_Ptr);
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            PT_EXIT(Pt);
        }
        
        //receive rx data & request tx buffer
        RosTimerStop(APP_PACKET_DELAY_TIMER);
        AppData.RX_Length = ((ApiSocketReceiveIndType*)Mail)->BufferLength;
        AppData.RX_Ptr = ((ApiSocketReceiveIndType*)Mail)->BufferPtr;
        
        SendApiSocketGetTxBufferReq(COLA_TASK, pInst->SocketHandle, maxBufferTX);
        PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_GET_TX_BUFFER_CFM) && ((ApiSocketGetTxBufferCfmType*)Mail)->Handle == pInst->SocketHandle);
        AppData.TX_Ptr = ((ApiSocketGetTxBufferCfmType*)Mail)->BufferPtr;
        AppData.TX_Length = maxBufferTX;
            
        
        //parse data (get method & urfile)
        parser_webdata(AppData.RX_Ptr);
        
        if (AppData.method == POST) {
            //prepare data for check cmd
            rsuint8 url_length = RSMIN(strlen((rschar*)AppData.ptr_url), AppData.TX_Length-1);
            memcpy(AppData.TX_Ptr, AppData.ptr_url, strlen((rschar*)AppData.ptr_url));
            memset(AppData.TX_Ptr+url_length,0x00,1);
            AppData.ptr_url = AppData.TX_Ptr;
            
            //start timer & request next data (vars)
            RosTimerStart(APP_PACKET_DELAY_TIMER, (300 * RS_T1MS), &PacketDelayTimer);
            SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, AppData.RX_Ptr);
            
            PT_YIELD_UNTIL(Pt, (IS_RECEIVED(API_SOCKET_RECEIVE_IND) && 
                    ((ApiSocketReceiveIndType*)Mail)->Handle == pInst->SocketHandle) || 
                    IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
            
            if (IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT)) {
                AppData.ptr_cmd = (rsuint8*)strstr((rschar*)(AppData.ptr_cmd+1), (rschar*)"\x0d\x0a\x0d\x0a");
                AppData.ptr_cmd += 4;
                
                rsuint8 l_tmp = AppData.RX_Ptr + AppData.RX_Length - AppData.ptr_cmd;
                l_tmp = RSMIN(l_tmp, sizeof(AppData.data)-1);
                memcpy(&AppData.data[0], AppData.ptr_cmd,l_tmp);
                AppData.data[l_tmp]=0x00;
                AppData.ptr_cmd = &AppData.data[0];
                
            } else {
                printf("Packet strange rcvd. Try kill connection...");
                while (1) {
                    if (IS_RECEIVED(API_SOCKET_RECEIVE_IND)) {
                        RosTimerStop(APP_PACKET_DELAY_TIMER);
                        RosTimerStart(APP_PACKET_DELAY_TIMER, (300 * RS_T1MS), &PacketDelayTimer);
                        SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, ((ApiSocketReceiveIndType*)Mail)->BufferPtr);
                        
                        PT_YIELD_UNTIL(Pt, (IS_RECEIVED(API_SOCKET_RECEIVE_IND) && 
                                ((ApiSocketReceiveIndType*)Mail)->Handle == pInst->SocketHandle) || 
                                IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT) );
                    }
                    else {
                        PT_EXIT(Pt);
                    }
                }
            }
            
            //find cmd for get/post methods
            find_var_cmd();
        }
        
        if (AppData.method == UNK) {
            //clear buffer
            SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, AppData.RX_Ptr); 
            
            //send 501 and exit
            AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template, "501 Not Implemented", "Method not implemented", web_error501);
            SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
            PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
            
            printf("HTTPD: 501 -> Method not implemented\n\n");
            RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
            PT_EXIT(Pt);
        }
        
        
        printf("HTTPD.URL: %s\n", (char*)AppData.ptr_url);        
        //clear rx buffer
        SendApiSocketFreeBufferReq(COLA_TASK, pInst->SocketHandle, AppData.RX_Ptr); 
        
        //if not exist cmd... check file in sdcard
        if (AppData.ptr_cmd == NULL) {
            //if file is '/', redirect to &HTTP_DEFAULT_WEBPAGE
            if (strlen((rschar*)AppData.ptr_url) == 1 ) {
                memcpy(AppData.ptr_url, &HTTP_DEFAULT_WEBPAGE[0], sizeof(HTTP_DEFAULT_WEBPAGE));
                printf("HTTPD: / -> Setting MAINPAGE -> %s\n", HTTP_DEFAULT_WEBPAGE);
            }
            
            //pre-check file (empty with &web_sdmask?) if not -> send 404
            if (strncmp(&web_sdmask[0],(rschar*)AppData.ptr_url,strlen(web_sdmask)) == 0) {
            } else {
                AppData.TX_Length = sprintf((rschar*)AppData.TX_Ptr, web_msg_template, "404 Not Found", "File not exist", web_error404);
                SendApiSocketSendReq(COLA_TASK, pInst->SocketHandle, AppData.TX_Ptr, AppData.TX_Length, 0);
                PT_YIELD_UNTIL(Pt, IS_RECEIVED(API_SOCKET_SEND_CFM) && ((ApiSocketSendCfmType*)Mail)->Handle == pInst->SocketHandle);
                RosMailSendP0(COLA_TASK, COLA_TASK, APP_WEBSERVER_IDLE);
                printf("HTTPD: Sended 404\n\n");
                PT_EXIT(Pt);
            }
            
            //download file and send it  
            PT_SPAWN(Pt, &childPt, Pt_SPI_getfile(&childPt, Mail, pInst, AppData.ptr_url));
            
        } else {
            AppData.ptr_cmd += sizeof(web_optioncmd)-1;            
            
            if (strncmp((rschar*)AppData.ptr_cmd, &web_optioncmd_RTX[0], strlen(web_optioncmd_RTX)) == 0) {
                //command is for RTX
                AppData.ptr_cmd += strlen(web_optioncmd_RTX);
                PT_SPAWN(Pt, &childPt, PT_process_rtx_command(&childPt, Mail, pInst, (rschar*)AppData.ptr_cmd));                
            } else {
                //command is for arduino (send via spi)
                PT_SPAWN(Pt, &childPt, Pt_SPI_sendcmd(&childPt, Mail, pInst, AppData.ptr_cmd));
            }
        }
    PT_END(Pt);
}

/**
 * @brief Init function for Web Server
 * @param PtList : pointer to Protothread control
 **/
PtEntryType* rtx_AppWebServer(RsListEntryType *PtList)
{
    ApiSocketAddrType addr;

    addr.Domain = ASD_AF_INET;
    addr.Ip.V4.Addr = 0;
    addr.Port = HTTP_SERVER_PORT;
 
printf ("HTTPD:Started\n\n");
    
    return AppSocketStartTcpServer(PtList, addr, PtOnHTTPreq);   
}
