/****************************************************************************
*  Program/file: rtx_AppWeb.h
*  
*  Copyright (C) by RTX A/S, Denmark.
*  These computer program listings and specifications, are the property of 
*  RTX A/S, Denmark and shall not be reproduced or copied or used in 
*  whole or in part without written permission from RTX A/S, Denmark.
*
*  Programmer: LKA
*
*  MODULE:
*  CONTROLLING DOCUMENT:
*  SYSTEM DEPENDENCIES:
*   
*   
*  DESCRIPTION:
*   
*   
*   
*   
*   
****************************************************************************/

/****************************************************************************
*                                  PVCS info                                 
*****************************************************************************

$Author:   Jordi Casas  $
$Date:   28 Apr 2015 07:19:38  $
$Revision:   1.0  $
$Modtime:   28 Apr 2015 07:19:38  $
$Archive:   ./Projects/Amelie/COLApps/Apps/SmartCitizenRTX4100/rtx_AppWeb.h  $

*/

/**
 @file rtx_AppWeb.h
 @brief Simple Web Server (header)
 @author Jordi Casas
 @date 4/2015, 5/2015, 6/2015
*/

#ifndef RTXAPPWEB_H
#define RTXAPPWEB_H

/****************************************************************************
*                               Include files                                 
****************************************************************************/
//Type definitions

//Framework/Kernel

//Interfaces

//Configurations

//Private 

/****************************************************************************
*                              Macro definitions                             
****************************************************************************/
#define HTTP_DEFAULT_WEBPAGE "/WEB/INDEX.HTM"
#define HTTP_SERVER_PORT 80

#define web_optioncmd "cmd="        /** Option for send the command */
#define web_optioncmd_RTX "rtx_"    /** Prefix for RTX function */
#define web_sdmask "/WEB/"          /** Prefix for file in SD */
#define web_cmd_separator "%3A"     /** Separator for parameters in cmd */

#define maxBufferTX 512             /** Max buffer for SPI*/
#define SPI_baudrate 8000000        /** Baudrate for SPI */ 



/****************************************************************************
*                     Enumerations/Type definitions/Structs                  
****************************************************************************/
/**
 * @brief Enum for supported methods
 **/
typedef enum {
    GET,    /**Get (vars in URI) */
    POST,   /**Post (vars in body) */
    UNK     /**Method not supported */
} type_method;

/**
 * @brief Struct for store info (app:AppWeb)
 **/
typedef struct { 
  rsuint8 *RX_Ptr;              /**Pointer to RX data */
  rsuint8 *TX_Ptr;              /**Pointer to TX data */
  rsuint16 RX_Length;           /**Length for RX data */
  rsuint16 TX_Length;           /**Length for TX data */
  rsuint8 data[256];            /**Buffer for temp data */
  type_method method;           /**Method of query */
  rsuint8 *ptr_url;             /**Pointer to url */
  rsuint8 *ptr_cmd;             /**Pointer to cmd var */
} AppDataWebType;

/****************************************************************************
*                           Global variables/const                           
****************************************************************************/
/** Template for send generic request. */
static const char web_msg_template[] = "HTTP/1.1 %s\r\n"
"Connection: close\r\n\r\n"
"<!DOCTYPE html><html><head><title>%s</title></head>"
"<body>%s</body></html>\r\n"; 

/** Template for send file. (Includes cache-control param in header) */
static const char web_msg_template_file[] = "HTTP/1.1 200\r\n"
"Connection: close\r\n"
"Cache-Control: public, max-age=3600\r\n"
"Content-Length: %i\r\n\r\n";

/** Template for send result cmd. (Includes link&timeout redirects) */
static const char web_msg_template_cmd[] = "HTTP/1.1 %s\r\n"
"Connection: close\r\n\r\n"
"<!DOCTYPE html><html><head><title>%s</title>"
"<meta name=\"viewport\" content=\"width=device-width,height=device-height,initial-scale=1.0\"/></head>"
"<body>Result command executed: <b>%s</b>"
"<br><br><a href='/'>Return to main page</a>"
"<meta http-equiv=\"Refresh\" content=\"10; url=/\">"
"</body></html>\r\n";

/** Template for file not exist */
static const char web_error404[] = "<h1>FILE NOT FOUND.</h1><br>Redirect in 3s to mainpage"
"<meta http-equiv=\"Refresh\" content=\"3; url=/\">";

/** Template for method not implemented */
static const char web_error501[] = "<h1>METHOD NOT IMPLEMENTED.</h1><br>Redirect in 3s to mainpage"
"<meta http-equiv=\"Refresh\" content=\"3; url=/\">";
/****************************************************************************
*                             Function prototypes                            
****************************************************************************/
PtEntryType* rtx_AppWebServer(RsListEntryType *PtList);

#endif

