/****************************************************************************
*  Program/file: DrvSpiSlave.h
*  
*  Copyright (C) by RTX A/S, Denmark.
*  These computer program listings and specifications, are the property of 
*  RTX A/S, Denmark and shall not be reproduced or copied or used in 
*  whole or in part without written permission from RTX A/S, Denmark.
*
*  Programmer: LKA
*
*  MODULE:
*  CONTROLLING DOCUMENT:
*  SYSTEM DEPENDENCIES:
*   
*  DESCRIPTION: Simple Slave Mode SPI driver. 
*               - Implemented for 8 bit data frames
*               - Support for 4 wire interface (CS active low)
*               - Implemented as SPI Mode 0 (CLKPOL = 0, CLKPHA = 0)
*               - Using USART2 of the MCU
*
****************************************************************************/

/****************************************************************************
*                                  PVCS info                                 
*****************************************************************************

$Author:   lka  $
$Date:   08 Mar 2013 10:31:02  $
$Revision:   1.0  $
$Modtime:   06 Mar 2013 19:43:20  $
$Archive:   J:/sw/Projects/Amelie/Components/Drivers/vcs/DrvSpiSlave.h_v  $

*/

#ifndef DrvSpiSlave_H
#define DrvSpiSlave_H

/****************************************************************************
*                               Include files                                 
****************************************************************************/
//Type definitions

//Framework/Kernel

//Interfaces

//Configurations

//Private 

/****************************************************************************
*                              Macro definitions                             
****************************************************************************/

/****************************************************************************
*                     Enumerations/Type definitions/Structs                  
****************************************************************************/
typedef struct 
{
  RosPrimitiveType Primitive;
  RsStatusType Status;
  rsuint8 *TxDataPtr; 
  rsuint16 TxDataLength;
} DrvSpiSlaveTxDoneMailType;

/****************************************************************************
*                           Global variables/const                           
****************************************************************************/


/****************************************************************************
*                             Function prototypes                            
****************************************************************************/

// This protothread will initialize the USART for SPI operation by calling DrvSpiSlaveInit().
PT_THREAD(PtDrvSpiSlaveInit(struct pt *Pt, const RosMailType* Mail, rsuint32 BaudRate));

// This protothread will close the SPI interface and restore original interrupt handling
PT_THREAD(PtDrvSpiSlaveClose(struct pt *Pt, const RosMailType* Mail));

// Driver init function. 
void DrvSpiSlaveInit(rsuint32 BaudRate);

// Get number of bytes stored in the RX buffer
rsuint16 DrvSpiSlaveRxGetSize(void);

// Read out data from the RX buffer
rsuint16 DrvSpiSlaveRx(rsuint8 *RxBufferPtr, rsuint16 RxBufferLength);

// Flush the RX buffer.
void DrvSpiSlaveRxFlush(void);

// Start TX 
RsStatusType DrvSpiSlaveTxStart(rsuint8 *TxDataPtr, rsuint16 TxDataLength);

// Set limit for full buffer message
void DrvSpiSlaveSetTrigger(rsuint16 l);

// Get max buffer usable
rsuint16 DrvSpiSlaveGetBufferSize();
#endif

